// CabUtil.java -- utilities for access cabinets

import java.io.*;
import java.util.*;

import org.okip.service.filing.api.*;
import org.okip.service.shared.api.FactoryManager;
import org.okip.service.shared.api.Type;

public class CabUtil
{
    private static CabinetFactory lfsFactory = null;
    
    public static Cabinet getCabinetFromDirectory(String path)
    {
        return getCabinetFromDirectory(new File(path));
    }
    
    /*
     *  Get a Cabinet given a directory in the local filesystem
     */
    public static Cabinet getCabinetFromDirectory(File directory)
    {
        if (!directory.isDirectory()) {
            errout("not a directory: " + directory);
            return null;
        }

        if (lfsFactory == null)
            lfsFactory = getLfsCabinetFactory();
        Cabinet cabinet = null;

        try {
            cabinet = lfsFactory.getCabinet(lfsFactory.idFromString(directory.getPath()));
        } catch (Exception e) {
            errout(e);
            return null;
        } 
        
        outln("Got dir [" + directory + "] thru cabinet [" + cabinet + "]");

        return cabinet;
    }

    /*
     *  Get a Local FilesSystem Cabinet Factory
     */
    private static CabinetFactory getLfsCabinetFactory()
    {
        try {
            Factory fileFactory = (Factory)
                FactoryManager.getDefaultFactory("org.okip.service.filing.api", null);
            outln("got defaultFileFactory [" + fileFactory + "]");

            CabinetFactory lfsFactory =
                fileFactory.getCabinetFactoryByType(new org.okip.service.filing.impl.localfilesystem.LfsType());
            outln("got lfsFactory by type [" + lfsFactory + "]");

            if (false) {
                // alternative method of getting a cabinet factory (one step vs two)
                // this doesn't work -- should it?
                // it ends up trying to find:
                //      org.okip.service.filing.impl.localfilesystem.Factory
                // when the class it wants is:
                //      org.okip.service.filing.impl.localfilesystem.CabinetFactory
                try {
                    Factory fileFactoryByName = (Factory)
                        FactoryManager.getFactory("org.okip.service.filing.api",
                                                  "org.okip.service.filing.impl.localfilesystem",
                                                  null);
                    outln("got fileFactory by name [" + fileFactoryByName + "]");
                } catch (Exception e) {
                    errout(e);
                }
            }
            
            return lfsFactory;
        
        } catch (Exception e) {
            errout(e);
            return null;
        }
    }
    
    /*
     * example code that gets a cabinet factory via capabilities
     * -- note that as written and with the current underlying implementation,
     * this method will currently return the local filesystem factory.
     */
    private static CabinetFactory getCabinetFactoryByProps()
    {
        try {
            CabinetFactory cabinetFactory = null;
        
            // get factory for implmentation
            Factory fileFactory = (Factory)
                FactoryManager.getDefaultFactory("org.okip.service.filing.api", null);
            // if I get this by TYPE, representable as a string, 
            
            java.util.Map desiredProperties = new java.util.Hashtable();
            desiredProperties.put(new CapabilityType("MIT", "readwrite"), new Boolean(true));
            CabinetFactory cabinetFactories[] = fileFactory.getCabinetFactory(desiredProperties);
            
            outln("got " + cabinetFactories.length + " cabinetFactory matching properties " + desiredProperties);
            
            if (cabinetFactories.length <= 0) {
                errout("unable to obtain CabinetFactory with properties [" + desiredProperties + "]");
                return null;
            }
            
            cabinetFactory = cabinetFactories[0];
            //outln("roots of cabinetFactory " + cabinetFactory + " are " + cabinetFactory.listRoots());
            outln("returning cabinetFactory " + cabinetFactory + " PROPS=" + cabinetFactory.getProperties());
            return cabinetFactory;
        
        } catch (Exception e) {
            errout(e.toString());
            return null;
        }
    }
    
    public static void printCabinet(Cabinet cabinet)
    {
        outln("The cabinet " + cabinet + " has the following entries:");
        Iterator i = cabinet.entries();
        while (i.hasNext())
            printCabinetEntry((CabinetEntry) i.next());
    }
    
    public static void printCabinetEntry(CabinetEntry ce)
    {
        printCabinetEntry(ce, System.out);
    }
    
    public static void printCabinetEntry(CabinetEntry ce, PrintStream ps)
    {
        ps.print((ce.isByteStore() ? " ByteStore " : " Cabinet   "));

        ps.print(ce.isCabinet() ? 'd' : '-');
        ps.print(ce.canRead() ? 'r' : '-');
        ps.print(ce.canWrite() ? 'w' : '-');
        ps.print(" [" + ce.getName() + "]");

        for (int i = 0; i < (12 - ce.getName().length()); i++)
            ps.print(' ');

        if (ce.isByteStore())
        {
            try {
                ByteStore bs = (ByteStore) ce;
                ps.print(new Date(bs.getLastModifiedTime()) + " ");
                for (int i = 100000; i > 1; i /= 10)
                    if (bs.length() < i) ps.print(' ');
                ps.print(" " + bs.length() + " bytes");
                // ps.print(bs.getOwner().getName()); unimplemented
                // ps.print(bs.getManifestAttributes()); no toString
                //ps.print(" MimeType " + bs.getMimeType());
                
//                 OkiInputStream is = bs.getOkiInputStream();
//                 byte[] b = new byte[16];
//                 int nRead = is.read(b);
//                 if (nRead >= 0) {
//                     ps.print("\tContents=["
//                         + formatForSource(new String(b, 0, nRead))
//                         + "]");
//                 }
                
            } catch (Exception e) {
                errout("error on " + ce + ": " + e);
                ps.println("error: " + ce + ": " + e);
            }
        }
        ps.println();
    }

    public static void parseCommandLine(String args[])
    {
        // handle setting look&feel if we can ought to
        // have this elsewhere as we don't actually want
        // to rely of having javax.swing available.
        String laf = null;
        for (int i = 0; i < args.length; i++) {
            String a = args[i];
            if (a.equals("-laf_java"))
                laf = javax.swing.UIManager.getCrossPlatformLookAndFeelClassName();
            else if (a.equals("-laf_windows"))
                laf = "com.sun.java.swing.plaf.windows.WindowsLookAndFeel";
        }
        if (laf != null) {
            try {
                outln("Setting look & feel to: " + laf);
                javax.swing.UIManager.setLookAndFeel(laf);
            } catch (Exception e) {
                errout(e);
            }
        }
    }
    
    private static void outln(String s) { System.out.println("CabUtil: " + s); }
    private static void errout(String s) { System.err.println("CabUtil: " + s); }
    private static void errout(Exception e)
    {
        errout(e.toString());
        e.printStackTrace();
    }
    

}
