//
//  CabViewer.java
//  S.Fraize July 2002
//

import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import java.text.*;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.tree.*;
import javax.swing.event.*;

import org.okip.service.filing.api.*;

public class CabViewer extends JFrame
    implements  ActionListener
{
    JTree jtree;
    JTable jtable;
    CabTreeNode rootNode;

    public CabViewer()
    {
        super("OKI Cabinet Viewer");
        debug("constructing CabViewer");
        
        this.setDefaultCloseOperation(EXIT_ON_CLOSE);
        this.setSize(new Dimension(800,640));
        this.setLocation(100,100);
        this.addMenus();
        this.setVisible(true);// show something right away
        
        /*
         * create the JTree
         */
        rootNode = new CabTreeNode("CabViewer(invisible root node)");
        rootNode.setDisplayName("CabViewer");

        this.jtree = new JTree(rootNode, true);
        this.jtree.setRootVisible(false);
        this.jtree.setExpandsSelectedPaths(true);
        addJTreeListeners();
        
        /*
         * create the JTable -- and we need a table-column model for that
         */
        CabinetTableModel cabinetDataModel = getDefaultCabinetTableModel();
        TableColumnModel columnModel = cabinetDataModel.getTableColumnModel();
        columnModel.setColumnMargin(10);// this makes for ugly breaks in line-selection indicator tho...
        this.jtable = new JTable(cabinetDataModel, columnModel);
        this.jtable.setGridColor(Color.lightGray);
        addTableRenderers();
        
        /*
         * create the splitPane & subcomponents with scrollers
         */
        JSplitPane splitPane = new JSplitPane();
        JScrollPane leftScroller = new JScrollPane(this.jtree);
        JScrollPane rightScroller = new JScrollPane(this.jtable);

        splitPane.setResizeWeight(0.25); // 25% space to the left component
        splitPane.setContinuousLayout(true);
        splitPane.setOneTouchExpandable(true);
        splitPane.setLeftComponent(leftScroller);
        splitPane.setRightComponent(rightScroller);

        Container pane = getContentPane();
        pane.setLayout(new BorderLayout());
        //pane.add(getToolBar(), BorderLayout.NORTH);
        pane.add(splitPane, BorderLayout.CENTER);
        
        this.constructing = false;
    }

    public void addRootCabinet(Cabinet cabinet)
    {
        CabTreeNode ctn = new CabTreeNode(cabinet);
        ctn.setDisplayName(cabinet.getPath());
        rootNode.add(ctn);
        ((DefaultTreeModel)jtree.getModel()).reload(rootNode);
        //jtree.expandPath(ctn.getPath());
    }

    CabinetTableModel defaultTableModel;
    CabinetTableModel getDefaultCabinetTableModel()
    {
        if (defaultTableModel == null)
            defaultTableModel = new CabinetTableModel(null);
        return defaultTableModel;
    }

    private JToolBar getToolBar()
    {
        // cache this!
        JToolBar tb = new JToolBar();
        //JButton tbQuit = new JButton("Quit");
        JButton tbClose = new JButton("Close");
        JButton tbRefresh = new JButton("Refresh");

        //tbQuit.addActionListener(this);
        tbClose.addActionListener(this);// will pick up "Close" action from menu
        tbRefresh.addActionListener(this);
        //tbQuit.setAction();

        //tb.add(tbQuit);
        tb.add(tbClose);
        tb.add(tbRefresh);

        
        tb.setBorderPainted(false);
        tb.setFloatable(false);
        return tb;
    }


    protected void addJTreeListeners()
    {
	jtree.addTreeSelectionListener
	    (new TreeSelectionListener() {
		    public void valueChanged(TreeSelectionEvent e)
		    {
			CabTreeNode cabNode = (CabTreeNode) jtree.getLastSelectedPathComponent();
			if (cabNode == null) return;
			debug("tree node selected: " + cabNode);
			jtable.setModel(cabNode.getDataModel());
		    }
		});

        // could put these on the the class instead of inline
        jtree.addTreeWillExpandListener
           (new TreeWillExpandListener() {
                   public void treeWillExpand(TreeExpansionEvent e)
                   {
                        TreePath path = e.getPath();
                        debug("tree expanding at path: " + path);
                        CabTreeNode cabNode = (CabTreeNode) path.getLastPathComponent();
			if (cabNode == null) return;
			debug("tree expanding at node: " + cabNode);
                        jtree.setSelectionPath(path);
                        jtable.setModel(cabNode.getDataModel());
                   }
                   public void treeWillCollapse(TreeExpansionEvent e) {}
               });

    }
                   

    protected void addTableRenderers()
    {
        // Set some renderers (defined in-line) for the CabinetEntry & Date classes.
        // Could also set these up by column instead of class in CabinetTableModel.createTableColumnModel();
        // It would probably be cleaner to do it there.

        jtable.setDefaultRenderer(CabinetEntry.class, new DefaultTableCellRenderer()
            { // in-line class definition

                // An example custom renderer -- if the CabinetEntry is unwriteable, make it red,
                // if a directory (Cabinet vs. ByteStore) make it blue and add a "/", if
                // emacs backup file, gray it out.
                public Component getTableCellRendererComponent(JTable table, Object value,
                                                               boolean isSelected, boolean hasFocus, int row, int column)
                 {
                     if (value instanceof CabinetEntry) {
                         CabinetEntry ce = (CabinetEntry) value;
                         String label = ce.getName();
                         if (ce.isCabinet()) {
                             label += "/";
                             setForeground(Color.blue);
                         }
                         else if (label.endsWith("~"))
                             setForeground(Color.gray);
                         else if (!ce.canWrite())
                             setForeground(Color.red);
                         else
                             setForeground(Color.black);
                         value = label;
                     }
                     return super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
                 }
            });
        
        jtable.setDefaultRenderer(Date.class, new DefaultTableCellRenderer()
            { // in-line class definition
                DateFormat dateFormatter = new SimpleDateFormat("MMM dd HH:mm:ss zzz yyyy");
                {
                    //setToolTipText("Last modification time");
                    setHorizontalAlignment(SwingConstants.RIGHT);
                }
                public Component getTableCellRendererComponent(JTable table, Object value,
                                                               boolean isSelected, boolean hasFocus, int row, int column)
                {
                    if (value instanceof Date) {
                        Date d = (Date) value;
                        value = dateFormatter.format(d);
                    } // else, it may be an empty string -- no date info available
                    return super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
                }
            });
    }
        
    
    private boolean constructing = true;
    private Font font = new Font("serif", Font.ITALIC+Font.BOLD, 48);
    public void paint(Graphics g)
    {
        super.paint(g);
        if (constructing) {
            g.setColor(Color.lightGray);
            g.setFont(font);
            g.drawString("OKI FILING DEMO", getWidth()/4, getHeight()/2);
        }
    }
    protected void _processEvent(AWTEvent e)
    {
        debug(e.toString());
	super.processEvent(e);
    }
    
	
    
    // Declarations for menus
    static JMenuBar mainMenuBar = null;
    static JMenu fileMenu = null;
    
    private static JMenuBar getMainMenuBar()
    {
        if (mainMenuBar == null)
            mainMenuBar = new JMenuBar();
        return mainMenuBar;
    }
    private static JMenu getFileMenu()
    {
        if (fileMenu == null)
            fileMenu = new JMenu("File");
        return fileMenu;
    }
        
    
    protected JMenuItem miNew;
    protected JMenuItem miOpen;
    protected JMenuItem miClose;
    protected JMenuItem miSave;
    protected JMenuItem miSaveAs;
	
    static final JMenu editMenu = new JMenu("Edit");
    protected JMenuItem miUndo;
    protected JMenuItem miCut;
    protected JMenuItem miCopy;
    protected JMenuItem miPaste;
    protected JMenuItem miClear;
    protected JMenuItem miSelectAll;
	
    public void addFileMenuItems() {
        getFileMenu();
        miNew = new JMenuItem ("New");
        miNew.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_N, java.awt.Event.META_MASK));
        fileMenu.add(miNew).setEnabled(false);
        miNew.addActionListener(this);

        miOpen = new JMenuItem ("Open...");
        miOpen.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_O, java.awt.Event.META_MASK));
        fileMenu.add(miOpen).setEnabled(false);
        miOpen.addActionListener(this);
		
        miClose = new JMenuItem ("Close");
        miClose.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_W, java.awt.Event.META_MASK));
        fileMenu.add(miClose).setEnabled(true);
        miClose.addActionListener(this);
		
        miSave = new JMenuItem ("Save");
        miSave.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_S, java.awt.Event.META_MASK));
        fileMenu.add(miSave).setEnabled(false);
        miSave.addActionListener(this);
		
        miSaveAs = new JMenuItem ("Save As...");
        fileMenu.add(miSaveAs).setEnabled(false);
        miSaveAs.addActionListener(this);
		
        getMainMenuBar().add(fileMenu);
    }
	
	
    public void addEditMenuItems() {
        miUndo = new JMenuItem("Undo");
        miUndo.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_Z, java.awt.Event.META_MASK));
        editMenu.add(miUndo).setEnabled(false);
        miUndo.addActionListener(this);
        editMenu.addSeparator();
 
        miCut = new JMenuItem("Cut");
        miCut.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_X, java.awt.Event.META_MASK));
        editMenu.add(miCut).setEnabled(false);
        miCut.addActionListener(this);

        miCopy = new JMenuItem("Copy");
        miCopy.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_C, java.awt.Event.META_MASK));
        editMenu.add(miCopy).setEnabled(false);
        miCopy.addActionListener(this);

        miPaste = new JMenuItem("Paste");
        miPaste.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_V, java.awt.Event.META_MASK));
        editMenu.add(miPaste).setEnabled(false);
        miPaste.addActionListener(this);

        miClear = new JMenuItem("Clear");
        editMenu.add(miClear).setEnabled(true);
        miClear.addActionListener(this);
        editMenu.addSeparator();

        miSelectAll = new JMenuItem("Select All");
        miSelectAll.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_A, java.awt.Event.META_MASK));
        editMenu.add(miSelectAll).setEnabled(true);
        miSelectAll.addActionListener(this);

        getMainMenuBar().add(editMenu);
    }
	
    public void addMenus() {
        addFileMenuItems();
        addEditMenuItems();
        //mainMenuBar.add(new JLabel("     Hello World    ", JLabel.RIGHT));
        //mainMenuBar.add(new JButton("Spin"));
        setJMenuBar (mainMenuBar);
    }

    // ActionListener interface (for menus)
    public void actionPerformed(ActionEvent newEvent) {
        if (newEvent.getActionCommand().equals(miNew.getActionCommand())) doNew();
        else if (newEvent.getActionCommand().equals(miOpen.getActionCommand())) doOpen();
        else if (newEvent.getActionCommand().equals(miClose.getActionCommand())) doClose();
        else if (newEvent.getActionCommand().equals(miSave.getActionCommand())) doSave();
        else if (newEvent.getActionCommand().equals(miSaveAs.getActionCommand())) doSaveAs();
        else if (newEvent.getActionCommand().equals(miUndo.getActionCommand())) doUndo();
        else if (newEvent.getActionCommand().equals(miCut.getActionCommand())) doCut();
        else if (newEvent.getActionCommand().equals(miCopy.getActionCommand())) doCopy();
        else if (newEvent.getActionCommand().equals(miPaste.getActionCommand())) doPaste();
        else if (newEvent.getActionCommand().equals(miClear.getActionCommand())) doClear();
        else if (newEvent.getActionCommand().equals(miSelectAll.getActionCommand())) doSelectAll();
        else
            debug("actionPerformed: " + newEvent.toString());
    }

    public void doNew() {}
    public void doOpen() { debug("doOpen"); }
    public void doClose() { debug("doClose"); System.exit(0); }
    public void doSave() {}
    public void doSaveAs() {}
    public void doUndo() {}
    public void doCut() { debug("cut row " + jtable.getSelectedRow()); }
    public void doCopy() {}
    public void doPaste() {}
    public void doClear() { jtable.clearSelection(); }
    public void doSelectAll() { jtable.selectAll(); }


    class CabTreeNode extends DefaultMutableTreeNode
    {
	AbstractTableModel dataModel;
	String displayName;
	
	CabTreeNode(Cabinet cabinet)
	{
	    super(cabinet);
	}
        
	private CabTreeNode(String s)
	{
	    super(s);
            setDisplayName(s);
            // for constructing the invisible root node
	}
        
        public void setDisplayName(String name)
        {
            this.displayName = name;
        }

	private Cabinet getCabinet()
	{
            if (getUserObject() instanceof Cabinet)
                return (Cabinet) getUserObject();
            return null;
	}
	
	AbstractTableModel getDataModel()
	{
	    if (this.dataModel == null) {
                debug("CabTreeNode: creating new CabinetTableModel for " + this.getCabinet());
		this.dataModel = new CabinetTableModel(this.getCabinet(), this);
            }
	    return this.dataModel;
	}
	
	/*
	 * toString is very functional here -- it's
	 * used to produce the text that the tree widget displays.
	 */
	public String toString()
	{
	    if (displayName != null)
		return displayName;
	    try {
		CabinetEntry ce = (CabinetEntry) userObject;
		return ce.getName();
	    } catch (Exception e) {
		errout(e);
		return userObject.getClass().toString();
	    }
	}
    }

    class CabinetTableModel extends AbstractTableModel
    {
        final int COL_NAME = 0;
        final int COL_SIZE = 1;
        final int COL_MODTIME = 2;
        final int COL_PERMISSIONS = 3;
        final int COL_OWNER = 4;

        Cabinet cabinet;
	ArrayList entries;
	CabTreeNode cabinetNode;
        int columnCount = 0;
	
        CabinetTableModel(Cabinet c)
	{
	    this(c, null);
	}
        CabinetTableModel(Cabinet c, CabTreeNode node)
	{
	    this.cabinetNode = node;
	    this.entries = new ArrayList();
            if (c != null)
                this.refreshModelFromCabinet(c, true);
	}
        
        /*
         * At the moment, getTableColumnModel and it's helper addTablecolumn
         * could really be methods on CabViewer (or static inner classes
         * if that were allowed), tho we're keeping them here and dynamic
         * in case we end up using different TableColumnModels for specific
         * cabinet types (or, perhaps, just via configuration preferences --
         * e.g., remembering a custom view on a particular cabinet).
         */
          
        TableColumnModel columnModel;
        public TableColumnModel getTableColumnModel()
        {
            if (columnModel != null)
                return columnModel;

            columnModel = new DefaultTableColumnModel();

            addTableColumn(COL_NAME, "File Name", 1000);
            addTableColumn(COL_SIZE, "Size", 200);
            addTableColumn(COL_MODTIME, "Modified", 700);
            addTableColumn(COL_PERMISSIONS, "Access", 150);
            //addTableColumn(COL_OWNER, "Owner", 500);

            return columnModel;
        }
        
        private void addTableColumn(int index, String name, int width)
        {
            TableColumn tc = new TableColumn(index, width);
            tc.setHeaderValue(name);
            columnCount++;
            columnModel.addColumn(tc);
        }
        
	public Class getColumnClass(int col) {
	    if (col == COL_NAME) return CabinetEntry.class;
	    if (col == COL_SIZE) return Integer.class;	        // this will cause align-right
	    if (col == COL_MODTIME) return Date.class;
 	    return super.getColumnClass(col);
	}

        public int 	getColumnCount() { return columnCount; }
        public int 	getRowCount() { return entries.size(); }
        public Object 	getValueAt(int row, int col)
	{
	    CabinetEntry ce = (CabinetEntry) entries.get(row);
	    return getColumnData(ce, col);
	}
	
	private Object getColumnData(CabinetEntry ce, int col)
	{
	    boolean isBS = ce.isByteStore();
	    boolean isCAB = ce.isCabinet();
	    ByteStore bs = isBS ? (ByteStore) ce : null;
	    Object data = null;

	    try {
		if (col ==  COL_NAME) {
                    data = ce;
                } else if (col == COL_SIZE) {
		    if (isBS)
                        data = new Long(bs.length());
                    else
                        data = "";
                } else if (col == COL_MODTIME) {
                    if (isBS)
                        data = new Date(bs.getLastModifiedTime());
                    else
                        data = "";
                } else if (col == COL_PERMISSIONS) {
		    String s = " ";
                    s += isCAB ? 'd' : '-';
		    s += ce.canRead() ? 'r' : '-';
		    s += ce.canWrite() ? 'w' : '-';
		    data = s;
                } else if (col == COL_OWNER) {
		    if (isBS) data = bs.getOwner();
		}
	    } catch (Exception e) {
		errout(e.toString());
		data = "<" + e.getMessage() + ">";
	    }
	    return data == null ? "-" : data;
	}

//         public String getColumnName(int col) {
// 	    if (col < columnNames.length)
// 		return columnNames[col];
// 	    return "";
// 	}
        
        
	public void refreshModelFromCabinet(Cabinet c)
        {
            refreshModelFromCabinet(c, false);
        }
            
	private void refreshModelFromCabinet(Cabinet c, boolean buildTree)
	{
            this.cabinet = c;
	    this.entries.clear();
	    Iterator i = this.cabinet.entries();
	    while (i.hasNext()) {
		CabinetEntry ce = (CabinetEntry) i.next();
		//debug("CabinetTableModel adding " + ce);
		entries.add(ce);

		// TODO: need to allow for updating the tree model also --
		// will need to clear & reset children.
                // Also: do we really want to do this here?
		if (buildTree && ce.isCabinet() && cabinetNode != null) {
		    this.cabinetNode.add(new CabTreeNode((Cabinet)ce));
		}
	    }
	}

    };
    //------------------------------------------------------------------
    // end of inner classes
    //------------------------------------------------------------------

    private static void out(String s) { System.out.print(s); }
    private static void outln(String s) { System.out.println("CabViewer: " + s); }
    private static void errout(String s) { System.err.println("CabViewer: " + s); }
    private static void errout(Exception e)
    {
        errout(e.toString());
        e.printStackTrace();
    }
    static void debug(String s) { if (A_DEBUG) System.out.println("CabViewer: " + s); }
    static boolean A_DEBUG = false;
    static boolean A_VERBOSE = false;
    public static void main(String args[])
    {
        CabUtil.parseCommandLine(args);
        String directoryName = ".";
        for (int i = 0; i < args.length; i++) {
            String a = args[i];
            if (a.equals("-debug")) {
                A_DEBUG = !A_DEBUG;
                if (A_DEBUG)
                    A_VERBOSE = true;
            } else if (a.equals("-verbose"))
                A_VERBOSE = !A_VERBOSE;
            else if (!a.startsWith("-"))
                directoryName = a;
        }
        
        try {
            Cabinet cabinet = CabUtil.getCabinetFromDirectory(directoryName);
            if (cabinet == null)
                System.exit(1);
            CabViewer cv = new CabViewer();
            cv.addRootCabinet(cabinet);
            //cv.addRootCabinet(CabUtil.getCabinetFromDirectory("."));
            cv.setVisible(true);
        } catch (Exception e) {
            errout(e);
            System.exit(2);
        }
    }
}
