
// SyncList.java
// MIT OKI Filing Demo
// S.Fraize July 2002

import java.util.*;
import java.io.*;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;

import org.okip.service.filing.api.*;


public class SyncList extends Hashtable
                              
{
    public static final int STATUS_UNKNOWN = 0;        // unknown status
    public static final int STATUS_CURRENT = 1;        // everything is copacetic
    public static final int STATUS_NEW_LOCAL = 2;      // new local entry, no remote data yet
    public static final int STATUS_CHANGED_LOCAL = 3;  // local version more current than remote version
    public static final int STATUS_NEW_REMOTE = 4;     // new remote entry, no local data yet
    public static final int STATUS_CHANGED_REMOTE = 5; // remote version more current than local version
    public static final int STATUS_CONFLICT = 6;       // versions in conflict -- both changed since last sync

    static final String PropsFileName = ".cabsync";
    
    Cabinet localCabinet;
    Cabinet remoteCabinet;
    SyncList remoteList;
    Properties props = new Properties();
    boolean isLocalList;
    ByteStore propsBS;

    boolean takeAction = true;
    boolean verbose = false;
    boolean debug = false;
    
    public SyncList(Cabinet localCab, Cabinet remoteCab)
    {
        this.localCabinet = localCab;
        this.remoteCabinet = remoteCab;
        this.isLocalList = true;
        this.readProperties();
        this.remoteList = new SyncList();
    }
    
    private SyncList() { this.isLocalList = false; }

    //rename
    private void setEntryProperty(SyncEntry se, String prop)
    {
        props.setProperty(se.getName(), prop);
    }

    private String getEntryProperty(SyncEntry se)
    {
        return props.getProperty(se.getName());
    }

    private void readProperties()
    {
        this.propsBS = getPropsByteStore();
        if (propsBS == null)
            errout("coudn't find a properties store in: " + this);
        else {
            try {
                this.props.load(propsBS.getOkiInputStream().getNativeInputStream());
                debug("loaded properties from BS " + propsBS);
            } catch (Exception e) {
                errout("failed to read properties from BS " + propsBS);
            }
        }
    }
        
    private void saveProperties()
    {
        // TODO: cull out dead entries -- if there's no CabinetEntry
        // for a name, delete it.
            
        String header = " CabSync properties, entries=(" + this.size() + "). ";
        header += "\n#  Local cabinet is " + getLocalCabinet();
        header += "\n# Remote cabinet is " + getRemoteCabinet();
        try {
            this.props.store(propsBS.getOkiOutputStream().getNativeOutputStream(), header);
        } catch (Exception e) {
            errout("Couldn't save properties store " + propsBS);
        }
    }
        
    private ByteStore getPropsByteStore()
    {
        ByteStore bs = null;

        try {
            bs = (ByteStore) localCabinet.getCabinetEntry(PropsFileName);
        } catch (NotFoundException e) {
            try {
                bs = localCabinet.createByteStore(PropsFileName);
                debug("created properties BS " + bs);
            } catch (Exception ex) {
                bs = null;
            }
        } catch (Exception e) {
            errout("couldn't get props file: " + e);
        }
        debug("got properties BS " + bs);
        return bs;
    }

    public Cabinet getLocalCabinet()
    {
        return localCabinet;
    }

    public Cabinet getRemoteCabinet()
    {
        return remoteCabinet;
    }

    private void scanCabinet(Cabinet c)
    {
        this.clear();
        Iterator i = c.entries();
        while (i.hasNext()) {
            SyncEntry se = new SyncEntry((CabinetEntry) i.next());

            if (this.isLocalList) {
                String dateStr = getEntryProperty(se);
                if (dateStr != null)
                    se.setLastSyncTime(Date.parse(dateStr));
            }
                
            if (se.getName().equals(PropsFileName)) // skip the properties file
                continue;
            this.put(se.getName(), se);
            debug("added syncEntry " + se);
        }
    }

    public void computeListStatus()
    {
        computeListStatus(false);
    }

    synchronized public void computeListStatus(boolean debug)
    {
        this.debug = debug;
        debug("SyncList:computeListStatus");

        // read in all the cabinet entries
        this.scanCabinet(getLocalCabinet());
        remoteList.scanCabinet(getRemoteCabinet());

        /*
         * Go through local file list, determining status.
         */
            
        Enumeration e = this.elements();
        while (e.hasMoreElements())
        {
            SyncEntry localEntry = (SyncEntry) e.nextElement();
            if (localEntry.isPlaceHolder())
                continue;
            SyncEntry remoteMatch = (SyncEntry) this.remoteList.get(localEntry.getName());
            debug("CMPL: " + localEntry);
            debug("CMPR: " + remoteMatch);
            int status = computeStatus(localEntry, remoteMatch);
            localEntry.setStatus(status);
            debug("\tstatus=" + localEntry.getStatusName());
            
            // TODO: keep a running entry of # of bytes to upload / download
            // might be easiest to do that in computeCompairedStatus?
            // OPTION: (advanced) boolean on add in byte total for entries that are in conflict
        }

        /*
         * Go through REMOTE file list, looking for
         * any new entries.
         */
            
        e = this.remoteList.elements();
        while (e.hasMoreElements()) {
            SyncEntry remoteEntry = (SyncEntry) e.nextElement();
            SyncEntry localMatch = (SyncEntry) this.get(remoteEntry.getName());
            if (localMatch == null) {
                // no match found, create new sync entry
                SyncEntry se = new SyncEntry(remoteEntry.getCabinetEntry());
                se.setIsPlaceHolder(true);
                se.setStatus(STATUS_NEW_REMOTE);
                this.put(remoteEntry.getName(), se);

                // TODO: rehack this where SyncEntrie's have ptrs to their
                // remove entry, which is just going to be damn convenient.
                // WARNING: This SyncEntry now has a bogus CabinetEntry --
                // it's from the REMOTE Cabinet, not the local...
                
            }
        }
        // doing a blanket notify for now regardless of
        // any changes -- computing status goes much faster
        // than actually coping data, so we can get away with this
        // for now.
        notifyChangeListeners(this);
    }

    /**
     * THIS IS THE MEAT WHERE SEMANATICS ARE WORKED OUT.
     *
     * Compare local & remote entries, returning computed
     * appropriate status for the local entry.
     * The remoteEntry is passed as a SyncEntry just for
     * convenience -- we're really just comparing the CabinetEntry instances.
     */
        
    protected int computeStatus(SyncEntry localEntry, SyncEntry remoteEntry)
    {
        if (remoteEntry == null)                // no remote was found under same name
            return STATUS_NEW_LOCAL;
        if (localEntry.isPlaceHolder())
            return STATUS_NEW_REMOTE;
        
        try {

            if (localEntry.isByteStore() && remoteEntry.isByteStore())
            {
                ByteStore localByteStore = localEntry.getByteStore();
                ByteStore remoteByteStore = remoteEntry.getByteStore();
                long localSyncTime = localEntry.getLastSyncTime();
                long localMTime = localByteStore.getLastModifiedTime();
                long remoteMTime = remoteByteStore.getLastModifiedTime();

                // if localSyncTime == 0, we've no last
                // sync time, and basically everything will
                // come up conflict (this will happen if you delete
                // the .cabsync (PropsFileName) file for example.
                // They're just going to have to delete everything and
                // start over, assuming remote as master.
                    
                boolean remoteChange = remoteMTime > localSyncTime;
                boolean localChange = localMTime > localSyncTime;
                    
                if (remoteChange && localChange)    return STATUS_CONFLICT;
                else if (localChange)               return STATUS_CHANGED_LOCAL;
                else if (remoteChange)              return STATUS_CHANGED_REMOTE;
                else if (localByteStore.length() != remoteByteStore.length()) {
                    // this should "never happen" -- the modification
                    // times are identical(?), but the sizes are different.
                    // Should we even bother checking for this?
                    errout("computeCompairedStatus: improbable condition: times identical, sizes different");
                    return STATUS_CONFLICT;
                }
                else
                    return STATUS_CURRENT;
            }
            // if both are CABINETS, status is going to wind up
            // unknown at the moment -- eventually cabinet status
            // will be derived from the sub-components, so unknown
            // is appropriate here.

            return STATUS_UNKNOWN;
                    
        } catch (Exception e) {
            errout("compairing " + localEntry + " " + e);
            return STATUS_UNKNOWN;
        }
    }
        
//     public void printStatus(PrintStream ps)
//     {
//         Enumeration e = this.elements();
//         while (e.hasMoreElements())
//             {
//                 SyncEntry se = (SyncEntry) e.nextElement();
//                 outln(se.toString());
//                 //printCabinetEntry(se);
//             }
//     }

    protected ArrayList listeners = new ArrayList();
    public void addChangeListener(ChangeListener cl)
    {
        listeners.add(cl);
    }

    public void notifyChangeListeners(Object src)
    {
        Iterator i = listeners.iterator();
        while (i.hasNext()) {
            ChangeListener cl = (ChangeListener) i.next();
            cl.stateChanged(new ChangeEvent(src));
        }
    }

    public void removeChangeListener(ChangeListener cl)
    {
        listeners.remove(listeners.indexOf(cl));
    }
    public void removeAllChangeListeners()
    {
        listeners.clear();
    }
        
    public void doSynchronize()
    {
        doSynchronize(true, false, false);
    }
    
    synchronized public void doSynchronize(boolean takeAction, boolean verbose, boolean debug)
    {
        this.takeAction = takeAction;
        this.verbose = verbose;
        this.debug = debug;
        
        debug("Synchronizing cabinets:");
        debug("\t (local) " + getLocalCabinet());
        debug("\t(remote) " + getRemoteCabinet());

        // TODO: Obtain a lock on the directories.
        // If there is currently a lock, we'll need
        // to re-compute the status when it's free
        // and try again.

        int changes = 0;
        Enumeration e = this.elements();
        while (e.hasMoreElements())
        {
            SyncEntry localEntry = (SyncEntry) e.nextElement();
            try {
                if (doSynchronizeEntry(localEntry)) {
                    localEntry.setStatus(STATUS_CURRENT);
                    changes++;
                    notifyChangeListeners(localEntry);
                }
            } catch (Exception ex) {
                errout(ex);
            }
        }
        if (takeAction) {
            this.saveProperties();
            this.readProperties();
        }
    }

    private boolean doSynchronizeEntry(SyncEntry localSyncEntry)
        throws FilingIOException, NameCollisionException, FilingPermissionDeniedException, IllegalCharacterException
               // TODO: handle these individually down below
    {
        boolean change = false;
        // TODO: how do we handle desired deletions?
        // would need to flag somehow that something was deliberately
        // deleted... This implies either a master source, who's
        // simple non-presence of an entry takes priority, or we
        // need a special utility to flagging this..
        int syncStatus = localSyncEntry.getStatus();
        statusOut(localSyncEntry.toString());
        switch (syncStatus) {
        case STATUS_UNKNOWN:
            // skip -- print anything?
            break;
        case STATUS_CURRENT:
            // just print a status line, nothing to do.
            break;
        case STATUS_NEW_LOCAL:
            // create new byteStore in remote cabinet
            try {
                copyCabinetEntry(getRemoteCabinet(), localSyncEntry.getCabinetEntry(), "create", "remote");
                if (takeAction) {
                    setLastSyncProperty(localSyncEntry, System.currentTimeMillis());
                    change = true;
                }
            } catch (Exception e) {
                errout(e);
            }
            break;
        case STATUS_CHANGED_LOCAL:
            // ship bytes to remote byteStore
            // TODO: should do a compare so if all bytes same, don't bother cascading updates
            // out to everyone who's listenting for changes.  Now, should we do the compare
            // locally or remotely?  (This also applies to STATUS_CHANGED_REMOTE)
            try {
                copyCabinetEntry(getRemoteCabinet(), localSyncEntry.getCabinetEntry(), "copy", "remote");
                if (takeAction) {
                    setLastSyncProperty(localSyncEntry, System.currentTimeMillis());
                    change = true;
                }
            } catch (Exception e) {
                errout(e);
            }
            break;
        case STATUS_NEW_REMOTE:
            // in this special case (syncStatys == STATUS_NEW_REMOTE)
            // and syncEntry.isPlaceHolder() == true, the
            // cabinet entry of the SyncEntry was pointed at
            // the REMOTE cabinet entry temporarily until we
            // synchronize it.  (this not ideal long term...)
                
            CabinetEntry remoteEntry = localSyncEntry.getCabinetEntry();

            // create new local entry to match the new remote
            try {
                CabinetEntry newEntry = copyCabinetEntry(getLocalCabinet(), remoteEntry, "create", "local");
                // restore the local SyncEntry to sanity with a real local cabinetEntry
                if (takeAction) {
                    localSyncEntry.setCabinetEntry(newEntry);
                    localSyncEntry.setIsPlaceHolder(false);
                    setLastSyncProperty(localSyncEntry, System.currentTimeMillis());
                    change = true;
                }
            } catch (Exception e) {
                errout(e);
            }
		    
            break;
        case STATUS_CHANGED_REMOTE:
            // grab bytes from remote byteStore
            try {
                CabinetEntry remoteCE = getRemoteCabinet().getCabinetEntry(localSyncEntry.getName());
                copyCabinetEntry(getLocalCabinet(), remoteCE, "copy", "local");
                if (takeAction) {
                    setLastSyncProperty(localSyncEntry, System.currentTimeMillis());
                    change = true;
                }
            } catch (Exception e) {
                errout(e);
            }
            break;
        case STATUS_CONFLICT:
            // inform the user...
            break;
        default:
            errout("SyncList:doSynchronize: unknown status: " + syncStatus);
        }

        return change;
    }

    private void setLastSyncProperty(SyncEntry se, long time)
    {
        se.setLastSyncTime(time);
        setEntryProperty(se, new Date(time).toString());
    }

    /**
     * @param destCabinet Cabinet we're creating the NEW CabinetEntry in
     * @param sourceEntry CabinetEntry we're copying
     * @param loc helpful string used only for verbose output (no function)
     *
     
     TODO: implement copy ourself so that we can progress-monitor it...
     InputStream in = new BufferedInputStream(
                          new ProgressMonitorInputStream(
                                  parentComponent,
                                  "Reading " + fileName,
                                  new FileInputStream(fileName)));
     */
    private CabinetEntry copyCabinetEntry(Cabinet destCabinet, CabinetEntry sourceEntry, String actionName, String loc)
        throws
	    NameCollisionException,		// NameCollision should be "impossible" at this point...
	    FilingIOException,
	    FilingPermissionDeniedException,
	    IllegalCharacterException
    {
        CabinetEntry newCabinetEntry = null;

        // Lfs filing BUG: this test doesn't seem to be working...
        if (sourceEntry.getParent().equals(destCabinet))
            errout("ack! incestous copy! cab=" + destCabinet + ", entry=" + sourceEntry);
	    
        if (sourceEntry.isByteStore()) {
            if (verbose)
                actionOut(actionName + " ByteStore '" + sourceEntry.getName() +
                          "' in/to " + loc + " cabinet " + destCabinet);
		
            if (takeAction) {
                ByteStore destBS = null;
                ByteStore sourceBS = (ByteStore) sourceEntry;
                try {
                    // we really just need a destCabinet.exists(name) call here...
                    destBS = (ByteStore) destCabinet.getCabinetEntry(sourceBS.getName());
                    if (destBS != null) {
                        if (debug) statusOut("deleting to make way: " + destBS);
                        destBS.delete();
                    }
                } catch (NotFoundException e) {
                    // it's not there, nothing to do, we're clear to create a new one.
                } catch (Exception e) {
                    errout(e);
                }

                newCabinetEntry = destCabinet.createByteStore(sourceBS.getName(), sourceBS);
                // verfification check...
                ByteStore newBS = (ByteStore) newCabinetEntry;
                if (newBS.length() != sourceBS.length())
                    throw new FilingIOException("copy failed! sizes differ:\n\t" + newBS + "\n\t" + sourceBS);
                //statusOut("bytestore states:\n\t src=" + sourceBS + "\n\tdest=" + newBS);
            }
        }
        else if (sourceEntry.isCabinet()) {
            if (verbose)
                actionOut("copy Cabinet '" + sourceEntry.getName() + "' in " + loc + " cabinet " + destCabinet);
            if (takeAction)
                destCabinet.createCabinet(sourceEntry.getName());
            // TODO: Need to recurse... somewhere.
        } else {
            errout(new Exception("copyCabinetEntry: neither ByteStore or Cabinet!"));
        }

        return newCabinetEntry;
    }

    public String toString()
    {
        return "SyncList:" + getLocalCabinet().toString();
    }

    private void debug(String s) { if (debug) System.out.println("SyncList: " + s); }
    private static void out(String s) { System.out.print(s); }
    private static void outln(String s) { System.out.println(s); }
    private static void errout(String s) { System.err.println("Error: " + s); }
    private static void errout(Exception e)
    {
        errout(e.toString());
        e.printStackTrace();
    }
    private void statusOut(String s) { if (verbose) outln("SyncList: " + s); }
    private void actionOut(String s) { if (verbose) outln("SyncList(" + (takeAction ? "Action): " : "Skipped): ") + s); }
    
    //-----------------------------------------------------------------------------
    // END OF SyncList
    //-----------------------------------------------------------------------------
}
