package org.okip.service.filing.api;

/*
   Copyright (c) 2002 Massachusetts Institute of Technology

   This work, including any software, documents, or other related items
   (the "Work"), is being provided by the copyright holder(s) subject to
   the terms of the MIT OKI(TM) API Definition License. By obtaining,
   using and/or copying this Work, you agree that you have read,
   understand, and will comply with the following terms and conditions of
   the MIT OKI(TM) API Definition License:

   You may use, copy, and distribute unmodified versions of this Work for
   any purpose, without fee or royalty, provided that you include the
   following on ALL copies of the Work that you make or distribute:

    *  The full text of the MIT OKI(TM) API Definition License in a
       location viewable to users of the redistributed Work.

    *  Any pre-existing intellectual property disclaimers, notices, or
       terms and conditions. If none exist, a short notice similar to the
       following should be used within the body of any redistributed
       Work: "Copyright (c) 2002 Massachusetts Institute of Technology. All
       Rights Reserved."

   You may modify or create Derivatives of this Work only for your
   internal purposes. You shall not distribute or transfer any such
   Derivative of this Work to any location or any other third party. For
   purposes of this license, "Derivative" shall mean any derivative of
   the Work as defined in the United States Copyright Act of 1976, such
   as a translation or modification.

   THIS WORK PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE WORK
   OR THE USE OR OTHER DEALINGS IN THE WORK.

   The name and trademarks of copyright holder(s) and/or MIT may NOT be
   used in advertising or publicity pertaining to the Work without
   specific, written prior permission. Title to copyright in the Work and
   any associated documentation will at all times remain with the
   copyright holders.

*/

/*
 * $Source: /cvs/oki/tech/src/org/okip/service/filing/api/ByteStore.java,v $
 */

/**
 * <p>
 * The ByteStore is the key interface in the Filing package.  It is a
 * set of bytes with associated physical metadata.
 * </p>
 * <p>
 * If the implementation supports it, each ByteStore may:
 * <ul>
 * <li>be replicated, stored redundantly on multiple servers for protection
 * against data loss.</li>
 * <li>it may have several versions of arbitrary depth</li>
 * </ul>
 * </p>
 * <p>
 * For ease of use, the ByteStore interface parallels when appropriate the
 * java.io.File class.
 * </p>
 * <p>
 * <b>Versioning</b>.  The default is to return
 * the latest version of a ByteStore.  Other versions may be accessed by a
 * key string.  Each version is itself a ByteStore, and may have versions
 * of its own, permitting branching.  The underlying principle is that
 * all versions should serve the same basic purpose, have the same
 * mime-type, be functionally equivalent.  If idiosyncrasies of a
 * particular version are significant, that version key may be
 * specified.  When a version key is specified, the base version rather
 * than any latest sub-version is returned.  Sub versions may then be
 * obtained if desired.  Different versions will only differ in bytes of
 * content and therefore length, lastModified time, and digest/checksum.
 * A particular version may be given a GUID only by creating a new
 * ByteStore in the Cabinet.
 * <p>
 * <b>Replication</b>.  If the implementation supports replication, all
 * writes wait until the bytes are confirmed to be stored on N servers.
 * Reads may use the most accessable server.
 * <p>
 * Licensed under the {@link org.okip.service.ApiLicense MIT OKI&#153; API Definition License}.
 *
 * @version $Name:  $ / $Revision: 1.11 $ / $Date: 2002/07/22 16:08:14 $
 */
public interface ByteStore
extends java.io.Serializable, Comparable, CabinetEntry {

  /**
   * Get ID of this ByteStore
   *
   * @return String ID
   *
   */
  public ID getID()
  throws FilingException;

  // from java.io.File

  /**
   * Returns the length of this ByteStore
   *
   * @return The length, in bytes, of this ByteStore
   *
   * @throws FilingIOException - if an IO error occurs reading Object
   */
  public long length()
  throws FilingException;

  /**
   * Returns the time that this ByteStore was last modified.
   *
   * @return  A <code>long</code> value representing the time the file was
   *          last modified, measured in milliseconds since the epoch
   *          (00:00:00 GMT, January 1, 1970)
   *
   * @throws FilingIOException - if an IO error occurs.
   */
  public long getLastModifiedTime()
  throws FilingException;

  /**
   * Sets the last-modified time of this ByteStore.
   *
   * <p> All platforms support file-modification times to the nearest second,
   * but some provide more precision.  The argument will be truncated to fit
   * the supported precision.  If the operation succeeds and no intervening
   * operations on the file take place, then the next invocation of the
   * <code>{@link #getLastModifiedTime}</code> method will return the (possibly
   * truncated) <code>time</code> argument that was passed to this method.
   *
   * @param  time  The new last-modified time, measured in milliseconds since
   *               the epoch (00:00:00 GMT, January 1, 1970)
   *
   * @return <code>true</code> if and only if the operation succeeded;
   *          <code>false</code> otherwise
   *
   * @throws FilingException - If the argument is negative
   * @throws FilingPermissionDeniedException - if Factory Owner
   * does not have permission to set the lastModified time.
   * @throws FilingIOException - if an IO error occurs setting the
   * lastModified time.
   */
  public boolean setLastModifiedTime(long time)
  throws FilingException;

  /**
   * Returns the time that this ByteStore was last accessed.
   *
   * Not all implementations will record last access times accurately,
   * due to caching and for performance.  The value returned will be
   * at least the last modified time, the actual time when a read was
   * performed may be later.
   *
   * @return  A <code>long</code> value representing the time the file was
   *          last accessed, measured in milliseconds since the epoch
   *          (00:00:00 GMT, January 1, 1970)
   *
   * @throws FilingIOException - if an IO error occurs.  */
  public long getLastAccessedTime()
  throws FilingException;

  /**
   * Returns the time that this ByteStore was created.
   *
   * Not all implementations will record the time of creation
   * accurately.  The value returned will be at least the last
   * modified time, the actual creation time may be earlier.
   *
   * @return  A <code>long</code> value representing the time the file was
   *          created, measured in milliseconds since the epoch
   *          (00:00:00 GMT, January 1, 1970)
   *
   * @throws FilingIOException - if an IO error occurs.
   */
  public long getCreatedTime()
  throws FilingException;

  /**
   * Deletes this ByteStore.
   *
   * @return  <code>true</code> if and only if this ByteStore is
   *          successfully deleted; <code>false</code> otherwise
   *
   * @throws FilingPermissionDeniedException - if Factory Owner
   * does not have permission to delete this ByteStore.
   * @throws FilingIOException - if an IO error occurs deleting the
   * ByteStore.
   */
  public boolean delete()
  throws FilingException;

  /* QUESTION: Should the byteStore be deleted from the Cabinets it
     is in, if any?  Can the ByteStore know what Cabinets it is in? */

  /**
   * Return owner of this ByteStore.
   *
   * Owner may be used for quota and/or access control.
   *
   * @return owner
   *
   * @throws FilingPermissionDeniedException - if Factory Owner
   * does not have permission to learn who owns this ByteStore.
   * @throws FilingIOException - if an IO error occurs.
   * @throws UnsupportedFilingOperationException - if owners of ByteStores
   * are not implemented.
   */
  public org.okip.service.shared.api.Agent getOwner()
  throws FilingException;

  /**
   * Set owner of this ByteStore.
   *
   * Owner may be used for quota and/or access control.
   *
   * @param owner
   *
   * @throws FilingPermissionDeniedException - if Factory Owner
   * does not have permission to change ownership of this ByteStore.
   * @throws FilingIOException - if an IO error occurs.
   * @throws UnsupportedFilingOperationException - if owners of ByteStores
   * are not implemented.
   */
  public void setOwner(org.okip.service.shared.api.Agent owner)
  throws FilingException;

  /**
   * Tests whether the Factory Owner may read this ByteStore.
   *
   * @return <code>true</code> if and only if this ByteStore can be
   *          read by the Factory Owner; <code>false</code> otherwise
   *
   */
  public boolean canRead()
  throws FilingException;

  /**
   * Marks this ByteStore so that only read operations are allowed.
   * After invoking this method this ByteStore is guaranteed not to
   * change until it is either deleted or marked to allow write
   * access.
   *
   * Note that whether or not a read-only ByteStore may be deleted
   * depends upon the underlying system of the implementation.
   *
   * @return <code>true</code> if and only if the operation succeeded;
   *          <code>false</code> otherwise
   *
   * @throws FilingPermissionDeniedException - if Factory Owner
   * does not have permission to set the ByteStore readOnly.
   * @throws FilingIOException - if an IO error occurs.
   */
  public boolean setReadOnly()
  throws FilingException;

  /**
   * Tests whether the Factory Owner may overwrite this ByteStore.  Note
   * that this does not include appending-only access.
   *
   * @return <code>true</code> if and only if the Factory Owner is
   *          allowed to overwrite this ByteStore
   *          <code>false</code> otherwise.
   *
   */
  public boolean canWrite()
  throws FilingException;

  /**
   * Marks this ByteStore so that write operations are allowed.
   *
   * @return <code>true</code> if and only if the operation succeeded;
   *          <code>false</code> otherwise
   *
   * @throws FilingPermissionDeniedException - if Factory Owner
   * does not have permission to set the ByteStore writable.
   * @throws FilingIOException - if an IO error occurs.
   * @throws UnsupportedFilingOperationException - if it is not
   * possible to set writable in this implementation
   */
  public boolean setWritable()
  throws FilingException;

  /**
   * Tests whether the Factory Owner may append to this ByteStore.
   *
   * @return <code>true</code> if and only if the Factory Owner is
   *          allowed to append to this ByteStore
   *          <code>false</code> otherwise.
   *
   */
  public boolean canAppend()
  throws FilingException;

  /**
   * Marks this ByteStore so that only append operations are allowed.
   *
   * @return <code>true</code> if and only if the operation succeeded;
   *          <code>false</code> otherwise
   *
   * @throws FilingPermissionDeniedException - if Factory Owner
   * does not have permission to set the ByteStore writable.
   * @throws FilingIOException - if an IO error occurs.
   * @throws UnsupportedFilingOperationException - if it is not
   * possible to restrict to append-only in this implementation
   */
  public boolean setAppendOnly()
  throws FilingException;

  /**
   * Tests whether the CabinetFactory implementation allows random
   * access to this ByteStore.
   *
   * @return <code>true</code> if and only if it is possible to use
   * random access operations on this ByteStore, <code>false</code>
   * otherwise.
   *
   */
  public boolean canSeek()
  throws FilingException;

  // added for OKI

  /**
   * Get the mime-type of this ByteStore.
   *
   * @return the mime-type (Content-Type in a jar file manifest)
   */
  public String getMimeType()
  throws FilingException;

  /**
   * Set the mime-type of this ByteStore.
   * <p>Returns the actual mime-type set for the ByteStore.  This may
   * differ from the supplied mime-type for several reasons.  The
   * implementation may not support the setting of the mime-type, in
   * which case the default mime-type or one derived from the content
   * bytes or file extension may be used.  Or a cannonical, IANA
   * mime-type (see
   * {@link "http://www.iana.org/assignments/media-types/index.html"} may be
   * substuted for a vendor or experimental type.
   *
   * @param mimeType
   *
   *
   * @return String
   * @throws FilingPermissionDeniedException - if Factory Owner
   * does not have permission to set the mime-type.
   * @throws FilingIOException - if an IO error occurs setting the
   * mime-type.
   */
  public String setMimeType(String mimeType)
  throws FilingException;

  /**
   * Returns the Digest using the specified algorithm used,
   * (algorithms such as md5 or crc).
   *
   * @param digestAlgorithm
   *
   * @return digest or null if unsupported
   *
   */
  public String getDigest(org.okip.service.shared.api.Type digestAlgorithm)
  throws FilingException;

  /**
   * Returns manifest attributes for this ByteStore.
   *
   *
   * @return java.util.jar.Attributes
   *
   */
  public java.util.jar.Attributes getManifestAttributes()
  throws FilingException;

  /**
   * Return OkiInputStream object which can be used to access this ByteStore.
   *
   * @return OkiInputStream object which may be used to access this
   * ByteStore.
   *
   * @throws FilingPermissionDeniedException - if Factory Owner
   * does not have permission to read this ByteStore.
   * @throws FilingIOException - if an IO error occurs.
   */
  public OkiInputStream getOkiInputStream()
  throws FilingException;

  /**
   * Return OkiSeekableInputStream object which can be used to access
   * this ByteStore.
   * <p>
   * The <code>canSeek()</code> method should be used to test whether
   * this is supported.  An UnsupportedFilingOperationException will
   * be thrown if seek is not supported in this CabinetFactory
   * implementation.
   * </p>
   * @return OkiSeekableInputStream object which may be used to access this
   * ByteStore.
   *
   * @throws FilingPermissionDeniedException - if Factory Owner
   * does not have permission to read this ByteStore.
   * @throws UnsupportedFilingOperationException - if seek is not
   * supported in this CabinetFactory implementation.
   * @throws FilingIOException - if an IO error occurs.  */
  public OkiSeekableInputStream getOkiSeekableInputStream()
  throws FilingException;

  /**
   * Return OkiOutputStream object which can be used to access this ByteStore.
   *
   * @return OkiOutputStream object which may be used to access this
   * ByteStore.
   *
   * @throws FilingPermissionDeniedException - if Factory Owner
   * does not have permission to read this ByteStore.
   * @throws FilingIOException - if an IO error occurs.
   */
  public OkiOutputStream getOkiOutputStream()
  throws FilingException;

  /**
   * Return OkiSeekableOutputStream object which can be used to access
   * this ByteStore.
   * <p>
   * The <code>canSeek()</code> method should be used to test whether
   * this is supported.  An UnsupportedFilingOperationException will
   * be thrown if seek is not supported in this CabinetFactory
   * implementation.
   * </p>
   * @return OkiSeekableOutputStream object which may be used to access this
   * ByteStore.
   *
   * @throws FilingPermissionDeniedException - if Factory Owner
   * does not have permission to read this ByteStore.
   * @throws UnsupportedFilingOperationException - if seek is not
   * supported in this CabinetFactory implementation.
   * @throws FilingIOException - if an IO error occurs.  */
  public OkiSeekableOutputStream getOkiSeekableOutputStream()
  throws FilingException;

  // versioning:

  /**
   * Return true if this ByteStore exists in multiple equivalent versions.
   *
   *
   * @return boolean
   *
   */
  public boolean hasVersions()
  throws FilingException;

  /**
   * Return latest version of this ByteStore.
   *
   *
   * @return ByteStore
   *
   */
  public ByteStore getLatestVersion()
  throws FilingException;

  /**
   * Return first version of this ByteStore.
   *
   *
   * @return ByteStore
   *
   */
  public ByteStore getBaseVersion()
  throws FilingException;

  /**
   * Return specified version of this ByteStore.
   *
   * @param versionKey
   *
   * @return ByteStore
   *
   * @throws UnsupportedFilingOperationException - if versions
   * are not implemented.
   */
  public ByteStore getVersion(java.lang.String versionKey)
  throws FilingException;

  /**
   * Return all versions of this ByteStore.
   *
   * @return ByteStore[]
   *
   */
  public ByteStore[] getVersions()
  throws FilingException;

  /**
   * Create new version for this ByteStore.
   *
   * @param agent
   * @param versionKey
   *
   * @return ByteStore
   *
   * @throws FilingPermissionDeniedException - if Factory Owner
   * does not have permission to create a new version of this ByteStore.
   * @throws FilingIOException - if an IO error occurs.
   * @throws UnsupportedFilingOperationException - if versions
   * are not implemented.
   */
  public ByteStore newVersion(org.okip.service.shared.api.Agent agent,
                              String versionKey)
  throws FilingException;

  /**
   * Compares this ByteStore to another Object.  If the object is not
   * a ByteStore, it throws a <code>ClassCastException</code>.
   *
   * @param o  The <code>Object</code> to be compared to ByteStore
   *
   * @return 0 if they are equal, less than 0 or greater than 0 if
   * they differ, depending on the implementation-dependent quality
   * used for the comparison.  If an error occurs, return less than 0.
   *
   * @see java.lang.Comparable
   */
  public int compareTo(Object o);

  /**
   * Compares this this ByteStore to another ByteStore.
   *
   * @param byteStore The <code>ByteStore</code> to be compared to
   * this ByteStore
   *
   * @return 0 if they are equal, less than 0 or greater than 0 if
   * they differ, depending on the implementation-dependent quality
   * used for the comparison.
   *
   * @throws FilingPermissionDeniedException - if Factory Owner
   * does not have read permission on the ByteStore being compared to
   * @throws FilingIOException - if an IO error occurs reading the
   * ByteStore being compared to
   *
   * @see java.lang.Comparable
   */
  public int compareTo(ByteStore byteStore)
  throws FilingException;

  /**
   * Tests this ByteStore for equality with the given object.
   * Returns <code>true</code> if and only if the argument is not
   * <code>null</code> and is equal to this ByteStore.
   *
   * @param object
   *
   * @return  <code>true</code> if and only if the objects are the same;
   *          <code>false</code> otherwise, or if an error occurs
   *
   */
  public boolean equals(Object object);

  /**
   * Computes a hash code for this ByteStore.
   *
   * @return  A hash code for this ByteStore
   */
  public int hashCode();

  /**
   * Returns the string representation of this ByteStore.
   *
   * @return  The string form of this ByteStore
   */
  public String toString();
}
