package org.okip.service.filing.impl.localfilesystem;

import org.okip.service.filing.api.*;


/**
   Copyright (c) 2002 Massachusetts Institute of Technology

   This work, including software, documents, or other related items (the
   "Software"), is being provided by the copyright holder(s) subject to
   the terms of the MIT OKI&#153; API Implementation License. By
   obtaining, using and/or copying this Software, you agree that you have
   read, understand, and will comply with the following terms and
   conditions of the MIT OKI&#153; API Implementation License:

   Permission to use, copy, modify, and distribute this Software and its
   documentation, with or without modification, for any purpose and
   without fee or royalty is hereby granted, provided that you include
   the following on ALL copies of the Software or portions thereof,
   including modifications or derivatives, that you make:

    *  The full text of the MIT OKI&#153; API Implementation License in a
       location viewable to users of the redistributed or derivative
       work.

    *  Any pre-existing intellectual property disclaimers, notices, or
       terms and conditions. If none exist, a short notice similar to the
       following should be used within the body of any redistributed or
       derivative Software:
       "Copyright (c) 2002 Massachusetts Institute of Technology
       All Rights Reserved."

    *  Notice of any changes or modifications to the MIT OKI&#153;
       Software, including the date the changes were made. Any modified
       software must be distributed in such as manner as to avoid any
       confusion with the original MIT OKI&#153; Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

   The name and trademarks of copyright holder(s) and/or MIT may NOT be
   used in advertising or publicity pertaining to the Software without
   specific, written prior permission. Title to copyright in the Software
   and any associated documentation will at all times remain with the
   copyright holders.

   The export of software employing encryption technology may require a
   specific license from the United States Government. It is the
   responsibility of any person or organization contemplating export to
   obtain such a license before exporting this Software.
*/

/*
 * $Source: /cvs/oki/tech/src/org/okip/service/filing/impl/localfilesystem/LfsByteStore.java,v $
 */

/**
 * Local file system ByteStore implementation.
 *
 * <p>
 * Licensed under the {@link org.okip.service.ApiImplementationLicenseMIT MIT OKI&#153; API Implementation License}.
 *
 * @version $Name:  $ / $Revision: 1.10 $ / $Date: 2002/07/22 17:04:21 $
 */

public class LfsByteStore
implements org.okip.service.filing.api.ByteStore {
  private java.io.File   file           = null;
  private LfsID          id             = null;
  private CabinetFactory cabinetFactory = null;

  // Constructor
  protected LfsByteStore(CabinetFactory cabinetFactory, String id)
  throws FilingPermissionDeniedException, FilingIOException,
         NotFoundException {

    //      System.out.println("creating byteStore: "+id);

    this.file           = new java.io.File(id);
    this.cabinetFactory = cabinetFactory;

    if (!file.isFile()) {
      throw new NotFoundException(id + " not found");
    }

    if (!file.canRead()) {
      throw new FilingPermissionDeniedException("cannot read " + id);
    }

    try {
      this.id = new LfsID(file.getCanonicalPath());
    }
    catch (Exception e) {
      throw new FilingIOException(e.getMessage());
    }
  }

  protected LfsByteStore(CabinetFactory cabinetFactory, String id,
                         boolean create)
  throws FilingPermissionDeniedException, FilingIOException {

    //      System.out.println("creating new byteStore: "+id);

    this.file           = new java.io.File(id);
    this.cabinetFactory = cabinetFactory;

    try {
      if (create) {
        if (!this.file.createNewFile()) {
          throw new FilingIOException(id + " could not be created");
        }
      }
      else {
        if (!this.file.isFile()) {
          throw new NotFoundException(id + " not found");
        }
      }

      if (!file.canRead()) {
        throw new FilingPermissionDeniedException("cannot read " + id);
      }

      this.id = new LfsID(file.getCanonicalPath());
    }
    catch (Exception e) {
      throw new FilingIOException(e.getMessage());
    }
  }

  /**
   * Get ID of this ByteStore
   *
   * @return ID
   *
   */
  public ID getID() {
    return this.id;
  }

  // from java.io.File

  /**
   * Returns the length of this ByteStore
   *
   * @return The length, in bytes, of this ByteStore
   *
   * @throws FilingIOException - if an IO error occurs reading Object
   */
  public long length()
  throws FilingIOException {
    long length = file.length();

    return length;

    /*
    if (length == 0) {
    // cannot tell the difference between 0 length and an error
        throw new FilingIOException("invalid length");
    } else {
    return length;
    }*/
  }

  /**
   * Returns the time that this ByteStore was last modified.
   *
   * @return  A <code>long</code> value representing the time the file was
   *          last modified, measured in milliseconds since the epoch
   *          (00:00:00 GMT, January 1, 1970)
   *
   * @throws FilingIOException - if an IO error occurs.
   */
  public long getLastModifiedTime()
  throws FilingIOException {
    long lastModified = file.lastModified();

    if (lastModified == 0) {
      throw new FilingIOException("invalid lastModified time");
    }
    else {
      return lastModified;
    }
  }

  /**
   * Sets the last-modified time of this ByteStore.
   *
   * <p> All platforms support file-modification times to the nearest second,
   * but some provide more precision.  The argument will be truncated to fit
   * the supported precision.  If the operation succeeds and no intervening
   * operations on the file take place, then the next invocation of the
   * <code>{@link #getLastModifiedTime}</code> method will return the (possibly
   * truncated) <code>time</code> argument that was passed to this method.
   *
   * @param  time  The new last-modified time, measured in milliseconds since
   *               the epoch (00:00:00 GMT, January 1, 1970)
   *
   * @return <code>true</code> if and only if the operation succeeded;
   *          <code>false</code> otherwise
   *
   * @throws FilingException - If the argument is negative
   * @throws FilingPermissionDeniedException - if Factory Owner
   * does not have permission to set the lastModified time.
   * @throws FilingIOException - if an IO error occurs setting the
   * lastModified time.
   */
  public boolean setLastModifiedTime(long time)
  throws FilingException, FilingPermissionDeniedException, FilingIOException {
    if (time < 0L) {
      throw new FilingException("IllegalArgumentException - negative time");
    }

    boolean success = file.setLastModified(time);

    if (!success) {
      if (!file.canWrite()) {
        throw new FilingPermissionDeniedException(
          "cannot set lastModifiedTime");
      }

      throw new FilingIOException("IO Error setting lastModifiedTime");
    }
    else {
      return true;
    }
  }

  /**
   * Returns the time that this ByteStore was last accessed.
   *
   * Not all implementations will record last access times accurately,
   * due to caching and for performance.  The value returned will be
   * at least the last modified time, the actual time when a read was
   * performed may be later.
   *
   * @return  A <code>long</code> value representing the time the file was
   *          last accessed, measured in milliseconds since the epoch
   *          (00:00:00 GMT, January 1, 1970)
   *
   * @throws FilingIOException - if an IO error occurs.  */
  public long getLastAccessedTime()
      throws FilingIOException {
      return this.getLastModifiedTime();
  }

  /**
   * Returns the time that this ByteStore was created.
   *
   * Not all implementations will record the time of creation
   * accurately.  The value returned will be at least the last
   * modified time, the actual creation time may be earlier.
   *
   * @return  A <code>long</code> value representing the time the file was
   *          created, measured in milliseconds since the epoch
   *          (00:00:00 GMT, January 1, 1970)
   *
   * @throws FilingIOException - if an IO error occurs.
   */
  public long getCreatedTime()
      throws FilingIOException {
      return this.getLastModifiedTime();
  }


  /**
   * Deletes this ByteStore.
   *
   * @return  <code>true</code> if and only if this ByteStore is
   *          successfully deleted; <code>false</code> otherwise
   *
   * @throws FilingPermissionDeniedException - if Factory Owner
   * does not have permission to delete this ByteStore.
   * @throws FilingIOException - if an IO error occurs deleting the
   * lastModified time.
   */
  public boolean delete()
  throws FilingIOException, FilingPermissionDeniedException {

    boolean success = file.delete();

    if (!success) {
      if (!file.canWrite()) {
        throw new FilingPermissionDeniedException("read only");
      }

      throw new FilingIOException("IO Error on deleting");
    }
    else {
      return true;
    }
  }

  /* QUESTION: Should the byteStore be deleted from the Cabinets it
     is in, if any?  Can the ByteStore know what Cabinets it is in? */


  /**
   * Return owner of this ByteStore.
   *
   * Owner may be used for quota and/or access control.
   *
   * @return owner
   *
   * @throws FilingPermissionDeniedException - if Factory Owner
   * does not have permission to learn who owns this ByteStore.
   * @throws FilingIOException - if an IO error occurs.
   * @throws UnsupportedFilingOperationException - if owners of ByteStores
   * are not implemented.
   */
  public org.okip.service.shared.api.Agent getOwner()
  throws FilingPermissionDeniedException, FilingIOException,
         UnsupportedFilingOperationException {
    throw new UnsupportedFilingOperationException(
      "ByteStore Ownership not implemented");
  }

  /**
   * Set owner of this ByteStore.
   *
   * Owner may be used for quota and/or access control.
   *
   * @param owner
   *
   * @throws FilingPermissionDeniedException - if Factory Owner
   * does not have permission to change ownership of this ByteStore.
   * @throws FilingIOException - if an IO error occurs.
   * @throws UnsupportedFilingOperationException - if owners of ByteStores
   * are not implemented.
   */
  public void setOwner(org.okip.service.shared.api.Agent owner)
  throws FilingPermissionDeniedException, FilingIOException,
         UnsupportedFilingOperationException {
    throw new UnsupportedFilingOperationException(
      "ByteStore Ownership not implemented");
  }

  /**
   * Tests whether the Factory Owner may read this ByteStore.
   *
   * @return <code>true</code> if and only if this ByteStore can be
   *          read by the Factory Owner; <code>false</code> otherwise
   *
   */
  public boolean canRead() {
    return this.file.canRead();
  }

  /**
   * Marks this ByteStore so that only read operations are allowed.
   * After invoking this method this ByteStore is guaranteed not to
   * change until it is either deleted or marked to allow write
   * access.
   *
   * Note that whether or not a read-only ByteStore may be deleted
   * depends upon the underlying system of the implementation.
   *
   * @return <code>true</code> if and only if the operation succeeded;
   *          <code>false</code> otherwise
   *
   * @throws FilingPermissionDeniedException - if Factory Owner
   * does not have permission to set the ByteStore readOnly.
   * @throws FilingIOException - if an IO error occurs.
   */
  public boolean setReadOnly()
  throws FilingPermissionDeniedException, FilingIOException {
    boolean success = file.setReadOnly();

    if (!success) {
      if (!file.canWrite()) {
        throw new FilingPermissionDeniedException(
          "cannot set lastModifiedTime");
      }

      throw new FilingIOException("IO Error setting ReadOnly");
    }
    else {
      return true;
    }
  }

  /**
   * Tests whether the Factory Owner may modify this ByteStore.
   *
   * @return <code>true</code> if and only if the Factory Owner is
   *          allowed to write to this ByteStore
   *          <code>false</code> otherwise.
   *
   */
  public boolean canWrite() {
    return this.file.canWrite();
  }

  /**
   * Marks this ByteStore so that write operations are allowed.
   *
   * @return <code>true</code> if and only if the operation succeeded;
   *          <code>false</code> otherwise
   *
   * @throws FilingPermissionDeniedException - if Factory Owner
   * does not have permission to set the ByteStore writable.
   * @throws FilingIOException - if an IO error occurs.
   * @throws UnsupportedFilingOperationException - if it is not
   * possible to set writable in this implementation
   */
  public boolean setWritable()
  throws FilingPermissionDeniedException, FilingIOException,
         UnsupportedFilingOperationException {
    throw new UnsupportedFilingOperationException(
      "Cannot set a ByteStore Writable in this implementation");
  }

  // not in java.io.File in 1.0-1.4

  /**
   * Tests whether the Factory Owner may append to this ByteStore.
   *
   * @return <code>true</code> if and only if the Factory Owner is
   *          allowed to append to this ByteStore
   *          <code>false</code> otherwise.
   *
   */
  public boolean canAppend() {
    return this.file.canWrite();
  }

  /**
   * Marks this ByteStore so that only append operations are allowed.
   *
   * @return <code>true</code> if and only if the operation succeeded;
   *          <code>false</code> otherwise
   *
   * @throws FilingPermissionDeniedException - if Factory Owner
   * does not have permission to set the ByteStore writable.
   * @throws FilingIOException - if an IO error occurs.
   * @throws UnsupportedFilingOperationException - if it is not
   * possible to restrict to append-only in this implementation
   */
  public boolean setAppendOnly()
  throws FilingPermissionDeniedException, FilingIOException,
         UnsupportedFilingOperationException {
    throw new UnsupportedFilingOperationException(
      "Cannot set a ByteStore AppendOnly in this implementation");
  }

  /**
   * Tests whether the CabinetFactory implementation allows random
   * access to this ByteStore.
   *
   * @return <code>true</code> if and only if it is possible to use
   * random access operations on this ByteStore, <code>false</code>
   * otherwise.
   *
   */
  public boolean canSeek() {
    return true;
  }

  // added for OKI

  /**
   * Get the mime-type of this ByteStore.
   *
   * @return the mime-type (Content-Type in a jar file manifest)
   */
  public String getMimeType() {

    // quick implementation.  Better implementation would have more
    // mime-types, configurable, and read the first few bytes of the
    // file as in the UNIX "file" command

    String name      = this.getName();
    String extension = name.substring(name.lastIndexOf(".") + 1);
    String mimeType  = (String) mimeTypesSuffixTable.get(extension);

    if (mimeType != null) {
      return mimeType;
    }
    else {
      return "application/octet-stream";
    }
  }

  private static java.util.Hashtable mimeTypesSuffixTable =
    new java.util.Hashtable();

  static {
    mimeTypesSuffixTable.put("doc", "application/msword");
    mimeTypesSuffixTable.put("pdf", "application/pdf");
    mimeTypesSuffixTable.put("ai", "application/postscript");
    mimeTypesSuffixTable.put("ps", "application/postscript");
    mimeTypesSuffixTable.put("eps", "application/postscript");
    mimeTypesSuffixTable.put("xls", "application/vnd.ms-excel");
    mimeTypesSuffixTable.put("ppt", "application/vnd.ms-powerpoint");
    mimeTypesSuffixTable.put("dcr", "application/x-director");
    mimeTypesSuffixTable.put("dir", "application/x-director");
    mimeTypesSuffixTable.put("dxr", "application/x-director");
    mimeTypesSuffixTable.put("swf", "application/x-shockwave-flash");
    mimeTypesSuffixTable.put("zip", "application/zip");

    //      mimeTypesSuffixTable.put("application/x-compress");
    mimeTypesSuffixTable.put("tar", "application/x-tar");
    mimeTypesSuffixTable.put("mpga", "audio/mpeg");
    mimeTypesSuffixTable.put("mp2", "audio/mpeg");
    mimeTypesSuffixTable.put("mp3", "audio/mpeg");
    mimeTypesSuffixTable.put("ram", "audio/x-pn-realaudio");
    mimeTypesSuffixTable.put("rm", "audio/x-pn-realaudio");
    mimeTypesSuffixTable.put("rpm", "audio/x-pn-realaudio");
    mimeTypesSuffixTable.put("ra", "audio/x-pn-realaudio");
    mimeTypesSuffixTable.put("wav", "audio/x-wav");
    mimeTypesSuffixTable.put("gif", "image/gif");
    mimeTypesSuffixTable.put("jpg", "image/jpeg");
    mimeTypesSuffixTable.put("jpeg", "image/jpeg");
    mimeTypesSuffixTable.put("jpe", "image/jpeg");
    mimeTypesSuffixTable.put("tif", "image/tiff");
    mimeTypesSuffixTable.put("tiff", "image/tiff");
    mimeTypesSuffixTable.put("bmp", "image/bmp");
    mimeTypesSuffixTable.put("html", "text/html");
    mimeTypesSuffixTable.put("htm", "text/html");
    mimeTypesSuffixTable.put("rtx", "text/richtext");
    mimeTypesSuffixTable.put("rtf", "text/rtf");
    mimeTypesSuffixTable.put("mpeg", "video/mpeg");
    mimeTypesSuffixTable.put("mpg", "video/mpeg");
    mimeTypesSuffixTable.put("mpe", "video/mpeg");
    mimeTypesSuffixTable.put("mov", "video/quicktime");
    mimeTypesSuffixTable.put("qt", "video/quicktime");
    mimeTypesSuffixTable.put("avi", "video/x-msvideo");
  }

  /**
   * Set the mime-type of this ByteStore.
   * <p>Returns the actual mime-type set for the ByteStore.  This may
   * differ from the supplied mime-type for several reasons.  The
   * implementation may not support the setting of the mime-type, in
   * which case the default mime-type or one derived from the content
   * bytes or file extension may be used.  Or a cannonical, IANA
   * mime-type (see 
   * {@link "http://www.iana.org/assignments/media-types/index.html"} may be
   * substuted for a vendor or experimental type.
   *
   * @param mimeType
   *
   *
   * @return String
   * @throws FilingPermissionDeniedException - if Factory Owner
   * does not have permission to set the mime-type.
   * @throws FilingIOException - if an IO error occurs setting the
   * mime-type.
   */
  public String setMimeType(String mimeType)
  throws FilingPermissionDeniedException, FilingIOException {

    // this implementation cannot set the mime-type, it derives it
    // from a guess based on the name
    return this.getMimeType();
  }

  /**
   * Returns the Digest using the specified algorithm used,
   * (algorithms such as md5 or crc).
   *
   * @param digestAlgorithm
   *
   * @return digest or null if unsupported
   *
   */
  public String getDigest(org.okip.service.shared.api.Type digestAlgorithm) {
    return null;
  }

  /**
   * Returns manifest attributes for this ByteStore.
   *
   *
   * @return java.util.jar.Attributes
   *
   */
  public java.util.jar.Attributes getManifestAttributes() {
    return null;
  }


  /**
   * Return OkiInputStream object which can be used to access this ByteStore.
   *
   * @return OkiInputStream object which may be used to access this
   * ByteStore.
   *
   * @throws FilingPermissionDeniedException - if Factory Owner
   * does not have permission to read this ByteStore.
   * @throws FilingIOException - if an IO error occurs.
   */
  public OkiInputStream getOkiInputStream()
      throws FilingPermissionDeniedException, FilingIOException {
      OkiInputStream ins = new LfsInputStream(this.file);

    return ins;
  }

  /**
   * Return OkiSeekableInputStream object which can be used to access
   * this ByteStore.
   * <p>
   * The <code>canSeek()</code> method should be used to test whether
   * this is supported.  An UnsupportedFilingOperationException will
   * be thrown if seek is not supported in this CabinetFactory
   * implementation.
   * </p>
   * @return OkiSeekableInputStream object which may be used to access this
   * ByteStore.
   *
   * @throws FilingPermissionDeniedException - if Factory Owner
   * does not have permission to read this ByteStore.
   * @throws UnsupportedFilingOperationException - if seek is not
   * supported in this CabinetFactory implementation.
   * @throws FilingIOException - if an IO error occurs.  */
  public OkiSeekableInputStream getOkiSeekableInputStream()
  throws FilingPermissionDeniedException, FilingIOException,
	 UnsupportedFilingOperationException {
      OkiSeekableInputStream ins = new LfsSeekableInputStream(this.file);

    return ins;
  }


  /**
   * Return OkiOutputStream object which can be used to access this ByteStore.
   *
   * @return OkiOutputStream object which may be used to access this
   * ByteStore.
   *
   * @throws FilingPermissionDeniedException - if Factory Owner
   * does not have permission to read this ByteStore.
   * @throws FilingIOException - if an IO error occurs.
   */
  public OkiOutputStream getOkiOutputStream()
      throws FilingPermissionDeniedException, FilingIOException {
      OkiOutputStream outs = new LfsOutputStream(this.file);

    return outs;
  }

  /**
   * Return OkiSeekableOutputStream object which can be used to access
   * this ByteStore.
   * <p>
   * The <code>canSeek()</code> method should be used to test whether
   * this is supported.  An UnsupportedFilingOperationException will
   * be thrown if seek is not supported in this CabinetFactory
   * implementation.
   * </p>
   * @return OkiSeekableOutputStream object which may be used to access this
   * ByteStore.
   *
   * @throws FilingPermissionDeniedException - if Factory Owner
   * does not have permission to read this ByteStore.
   * @throws UnsupportedFilingOperationException - if seek is not
   * supported in this CabinetFactory implementation.
   * @throws FilingIOException - if an IO error occurs.  */
  public OkiSeekableOutputStream getOkiSeekableOutputStream()
  throws FilingPermissionDeniedException, FilingIOException,
	 UnsupportedFilingOperationException{
      OkiSeekableOutputStream outs = new LfsSeekableOutputStream(this.file);

    return outs;
  }

  // versioning:

  /**
   * Return true if this ByteStore exists in multiple equivalent versions.
   *
   *
   * @return boolean
   *
   */
  public boolean hasVersions() {
    return false;
  }

  /**
   * Return latest version of this ByteStore.
   *
   *
   * @return ByteStore
   *
   */
  public ByteStore getLatestVersion() {
    return this;
  }

  /**
   * Return first version of this ByteStore.
   *
   *
   * @return ByteStore
   *
   */
  public ByteStore getBaseVersion() {
    return this;
  }

  /**
   * Return specified version of this ByteStore.
   *
   * @param versionKey
   *
   * @return ByteStore
   *
   * @throws UnsupportedFilingOperationException - if versions
   * are not implemented.
   */
  public ByteStore getVersion(java.lang.String versionKey)
  throws UnsupportedFilingOperationException {
    throw new UnsupportedFilingOperationException(
      "ByteStore versioning not implemented");
  }

  /**
   * Return all versions of this ByteStore.
   *
   * @return ByteStore[]
   *
   */
  public ByteStore[] getVersions() {
    ByteStore[] b = new ByteStore[1];

    b[0] = this;

    return b;
  }

  /**
   * Create new version for this ByteStore.
   *
   * @param agent
   * @param versionKey
   *
   * @return ByteStore
   *
   * @throws FilingPermissionDeniedException - if Factory Owner
   * does not have permission to create a new version of this ByteStore.
   * @throws FilingIOException - if an IO error occurs.
   * @throws UnsupportedFilingOperationException - if versions
   * are not implemented.
   */
  public ByteStore newVersion(org.okip.service.shared.api.Agent agent,
                              String versionKey)
  throws FilingPermissionDeniedException, FilingIOException,
         UnsupportedFilingOperationException {
    throw new UnsupportedFilingOperationException(
      "ByteStore versioning not implemented");
  }

  /**
   * Compares this ByteStore to another Object.  If the object is not
   * a ByteStore, it throws a <code>ClassCastException</code>.
   *
   * @param o  The <code>Object</code> to be compared to ByteStore
   *
   * @return 0 if they are equal, less than 0 or greater than 0 if
   * they differ, depending on the implementation-dependent quality
   * used for the comparison.  If an error occurs, return less than 0.
   *
   * @see java.lang.Comparable
   */
  public int compareTo(Object o) {
    try {
      return compareTo((ByteStore) o);
    }
    catch (ClassCastException e) {
      throw e;
    }
    catch (Exception e) {
      return -1000;
    }
  }

  /**
   * Compares this this ByteStore to another ByteStore.
   *
   * @param byteStore The <code>ByteStore</code> to be compared to
   * this ByteStore
   *
   * @return 0 if they are equal, less than 0 or greater than 0 if
   * they differ, depending on the implementation-dependent quality
   * used for the comparison.
   *
   * @throws FilingPermissionDeniedException - if Factory Owner
   * does not have read permission on the ByteStore being compared to
   * @throws FilingIOException - if an IO error occurs reading the
   * ByteStore being compared to
   *
   * @see java.lang.Comparable
   */
  public int compareTo(ByteStore byteStore)
  throws FilingException {
    return this.id.toString().compareTo(byteStore.getID().toString());
  }

  /**
   * Tests this ByteStore for equality with the given object.
   * Returns <code>true</code> if and only if the argument is not
   * <code>null</code> and is equal to this ByteStore.
   *
   * @param object
   *
   * @return  <code>true</code> if and only if the objects are the same;
   *          <code>false</code> otherwise, or if an error occurs.
   */
  public boolean equals(Object object) {
    if (object.getClass() != this.getClass()) {
      return false;
    }
    else {
      try {
        return this.id.toString().equals(
          ((ByteStore) object).getID().toString());
      }
      catch(Throwable ex) {
        return false;
      }
    }
  }

  /**
   * Computes a hash code for this ByteStore.
   *
   * @return  A hash code for this ByteStore
   */
  public int hashCode() {
    return this.file.hashCode();
  }

  /**
   * Returns the string representation of this ByteStore.
   *
   * @return  The string form of this ByteStore
   */
  public String toString() {
    return this.id.toString();
  }

  /**
   * Tests whether this is a Cabinet.
   *
   *
   * @return boolean - true if this entry is a Cabinet.
   *
   */
  public boolean isCabinet() {
    return false;
  }

  /**
   * Tests whether this is a ByteStore.
   *
   *
   * @return boolean - true if this entry is a ByteStore.
   *
   */
  public boolean isByteStore() {
    return true;
  }

  /**
   * Returns the Cabinet in which this is an entry, or null if it has
   * no parent (for example is the root cabinet).
   *
   *
   * @return Cabinet - the parent Cabinet of this entry, or null if it has
   * no parent (e.g. is the root cabinet)
   *
   * @throws FilingIOException - if an IO error occurs.
   * @throws FilingPermissionDeniedException - if Factory Owner
   * does not have read permission on the parent Cabinet.
   */
  public Cabinet getParent()
  throws FilingIOException, FilingPermissionDeniedException {
    try {
      java.io.File parentFile = this.file.getParentFile();

      if (parentFile != null) {
        String parentString = parentFile.getCanonicalPath();

        return new LfsCabinet(this.cabinetFactory, parentString);
      }
      else {
        return null;
      }
    }
    catch (Exception e) {
      throw new FilingIOException("Error getting parent:" + e.getMessage());
    }
  }

  /**
   * Return the name of this CabinetEntry in its parent Cabinet.
   *
   * @return name
   *
   */
  public String getName() {
    return this.file.getName();
  }

  /**
   * Return the path, the sequence of names of CabinetEntries leading
   * to this CabinetEntry from its root Cabinet, separated by the
   * separatorCharactor.
   *
   * @return path
   *
   */
  public String getPath() {
    return this.id.toString();
  }
}
