package org.okip.service.logging.api;



/**
Interface LogService

Purpose: Represents the key interface to the Logging subsystem. 
Maintains configuration information used by related entities. The LogService 
sets up and broadcasts messages to a collection of associated logging 
handlers that are called to log information.

LogService represents the logging service abstraction.

*/

public interface LogService
extends java.io.Serializable {

  /**
   * Method registerLog
   * Register a Log with the LogService under a specific logical name. 
   * Each log serves as a kind of handler for logging.
   *
   * @param logName Represents the logical name of the Log
	 * @param log			Represents the specific logging handler.
   *
   * @throws LogServiceException if some error occurs while linking this handler
   *
   */
  public void registerLog(final String logName, final Log log)
  throws LogServiceException;

  /**
   * Method deregisterLog
   * Deregister a log from the LogService.
   *
	 * @param log			Represents the specific logging handler.
   *
   * @throws LogServiceException if some error occurs while unlinking this 
	 * handler.
   *
   */
  public void deregisterLog(final Log log)
  throws LogServiceException;

  /**
   * Method deregisterLog
   * Deregister a log from the LogService.
   *
   * @param logName Represents the name under which a logging handler 
   * was previously registered.
   *
   * @throws LogServiceException if some error occurs while unlinking this 
   * handler.
   *
   */
  public void deregisterLog(final String logName)
  throws LogServiceException;

  /**
   * Method openLog
   * Opens  a registered log.
   *
   * @param who Represents the Agent for authn/authz purposes.
   * @param logName Represents the logical name of the Log
   * @param mode 		Represents the mode in which to open the log -- READ/APPEND
   *
   * @return Log
   * @throws LogServiceException if some error occurs while opening this Log.
   *
   */
  public Log openLog(org.okip.service.shared.api.Agent who,
                     final String logName, 
										 final org.okip.service.shared.api.Type mode)
  throws LogServiceException;

  /**
   * Method openLog
   * Opens  a registered log, allocating its resources..
   *
   * @param who Represents the Agent for authn/authz purposes.
	 * @param log			Represents the specific logging handler.
   * @param mode 		Represents the mode in which to open the log -- READ/APPEND
   *
   * @return Log
   * @throws LogServiceException if some error occurs while opening this Log.
   *
   */
  public Log openLog(org.okip.service.shared.api.Agent who,
                      final Log log, org.okip.service.shared.api.Type mode)
  throws LogServiceException;

  /**
   * Method closeLog
   * Closes  a registered log, freeing its resources.
   *
   * @param logName Represents the logical name under which the Log was opened.
   *
   * @throws LogServiceException if some error occurs while closing this Log.
   *
   */
  public void closeLog(final String logName)
  throws LogServiceException;

  /**
   * Method closeLog
   * Closes  a registered log, freeing its resources.
   *
	 * @param log			Represents the specific logging handler.
   * @param mode 		Represents the mode in which to open the log -- READ/APPEND
   * NB: This implies a Log may be open in one or both modes simultaneously?
   *
   * @throws LogServiceException if some error occurs while closing this Log.
   *
   */
  public void closeLog(final Log log, org.okip.service.shared.api.Type mode)
  throws LogServiceException;

  /**
   * Method closeLogs
   * Closes *all* registered logs, freeing their resources.
   *
   * @throws LogServiceException if some error occurs while closing any Log.
   *
   */
  public void closeLogs()
  throws LogServiceException;

  /**
   * Method getAvailablePriorities
   * Get all available logging Priority Levels.
   *
   * @return org.okip.service.logging.Priority[]
   *
   */
  public Priority[] getAvailablePriorities();

}
